<?php namespace xp\compiler\checks;

/**
 * Verifies method calls
 *
 * @test    xp://net.xp_lang.tests.checks.MethodCallVerificationTest
 */
class MethodCallVerification extends AbstractMethodCallVerification {

  /**
   * Return node this check works on
   *
   * @return  lang.XPClass<? extends xp.compiler.ast.Node>
   */
  public function node() {
    return \lang\XPClass::forName('xp.compiler.ast.MethodCallNode');
  }

  /**
   * Return whether this check is to be run deferred
   *
   * @return  bool
   */
  public function defer() {
    return true;
  }
  
  /**
   * Executes this check
   *
   * @param   xp.compiler.ast.Node node
   * @param   xp.compiler.types.Scope scope
   * @return  bool
   */
  public function verify(\xp\compiler\ast\Node $node, \xp\compiler\types\Scope $scope) {
    $call= \cast($node, 'xp.compiler.ast.MethodCallNode');

    // Verify type
    // * var: Might have method
    // * primitive, array, map, int: Definitely don't have methods
    $type= $scope->typeOf($call->target);
    if ($type->isVariable()) {
      return ['T203', 'Member call (var).'.$call->name.'() verification deferred until runtime'];
    } else if (!$type->isClass()) {
      return ['T305', 'Using member calls on unsupported type '.$type->compoundName()];
    }

    return $this->verifyMethod($type, $call->name, $scope);
  }
}
